;(function (document, $) {

    window.smartling = window.smartling || {};
    window.smartling.touchui = window.smartling.touchui || {};
    window.smartling.touchui.jobDialog = window.smartling.touchui.jobDialog || {};
    var dialog = window.smartling.touchui.jobDialog;

    dialog.openDialog = function(path) {
        dialog.setPayload(path)
        $('#smartling-job-dialog').get(0).show();
    }

    dialog.closeDialog = function () {
        $('#smartling-job-dialog').get(0).hide();
    }

    dialog.setPayload = function(payload) {
        if (payload) {
            $('input[name="payload"]').val(payload);
            dialog.pagePath = payload;
        }
    }

    dialog.initializeForm = function(action) {
        var foundationUi = $(window).adaptTo('foundation-ui');
        var $dialog = $('#smartling-job-dialog');
        var $tabs = $('#smartling-job-dialog-tabs');

        $dialog.find('form').each(function() {
           $(this).attr('action', action);
        });

        $dialog.find('coral-datepicker').each(function () {
            this.min = moment().startOf('day').add(1, 'days').toDate();
        });

        $dialog.find('form')
            .on('foundation-form-submitted', function (e, success) {
                if (success) {
                    dialog.closeDialog();
                    foundationUi.notify('Success', 'Page was added to a job', 'success');
                }
            })
            .on('reset', function () {
                this.querySelector('coral-checkbox[name="authorizeJob"]').checked = true;
                this.querySelector('coral-checkbox[name="isDeep"]') &&
                    (this.querySelector('coral-checkbox[name="isDeep"]').checked = false);
                this.querySelector('coral-checkbox[name="forceResubmission"]').checked = true;

                $('.smartling-touch-js-language-selector-field').each(function () {
                    var $selector = $(this);
                    var languageSelectorField = CUI.Widget.fromElement('Autocomplete', $selector);
                    languageSelectorField.reset();
                });
            });

        $('#smartling-job-select').on('coral-select:showitems', function (e) {
            var select = this;

            if (select.items.length === 0) {
                e.preventDefault();

                $.ajax({
                    url: '/services/smartling/jobs?path=' + dialog.pagePath
                }).done(function (data) {
                    data.forEach(function (job) {
                        var redableState;
                        if (job.state == 'AWAITING_AUTHORIZATION') {
                            redableState = '(Unauthorized)';
                        } else if (job.state == 'IN_PROGRESS') {
                            redableState = '(In progress)';
                        } else if (job.state == 'COMPLETED') {
                            redableState = '(Completed)'
                        }
                        else {
                            redableState = '(' + job.state + ')'
                        }
                        select.items.add({value: job.uid, content: {innerHTML: job.name + ' ' + redableState}});
                    });
                }).always(function () {
                    select.loading = false;
                });
            }
        }).on('change', function (e) {
            var form = $(this).closest('form').get(0);
            var dueDateFiled = form.querySelector('coral-datepicker[name="jobDueDate"]');
            var $targetLocalesField = $(form.querySelector('[name="targetLocaleCodes"]').closest('.coral-Form-field'));

            foundationUi.wait(form);
            $.ajax({
                url: '/services/smartling/jobs/detail?path=' + dialog.pagePath + '&uid=' + e.target.value
            }).done(function (data) {
                dueDateFiled.value = data.dueDate;
                var selections = [];
                data.targetLocales && data.targetLocales.forEach(function(elem) {
                    selections.push({
                        value: elem.code,
                        text: elem.label
                    })
                });
                $targetLocalesField[0]._setSelections(selections);
                $targetLocalesField.find(".coral-Form-fielderror").hide();
                $(".coral-Tab").removeAttr("invalid");
                $targetLocalesField.invalid = false;
            }).fail(function () {
                dueDateFiled.value = null;
                $targetLocalesField.find('[name="targetLocaleCodes"]').empty();
            }).always(function () {
                foundationUi.clearWait();
            });
        });

        $tabs.on('coral-panelstack:change', function() {
            var $submitButton = $('#smartling-job-dialog-submit');
            var $form = $(this.panelStack.selectedItem).find('form');
            $submitButton.attr('form', $form.attr('id'));

            $(".coral-Form-fielderror").hide();
            $(".coral-Form-field").removeAttr("invalid");
            $(".coral-Tab").removeAttr("invalid");
        });

        $tabs.find("coral-tab").on('click', function() {
            var $submitButton = $('#smartling-job-dialog-submit');
            if ($submitButton.length) {
                $submitButton.get(0).label.innerText = $(this).find("coral-tab-label").text();
            }
        });

        $dialog.on('coral-overlay:close', '*', function (e) {
            e.stopPropagation();
        }).on('coral-overlay:close', function () {
            var $forms = $(this).find('form');
            var PAGE_THUMBNAIL = 'cq-wcm-pagethumbnail';
            if ($forms.find(PAGE_THUMBNAIL).length === 0) {
                var $div = $('<div/>').addClass(PAGE_THUMBNAIL).hide();
                var $button = $('<button/>').attr({type:'reset'}).hide();
                $div.append($button);
                $forms.append($div);
            }

            $(this).find('form').each(function () {
                this.reset();
            });

            $(".coral-Form-fielderror").hide();
            $(".coral-Form-field").removeAttr("invalid");
            $(".coral-Tab").removeAttr("invalid");
            $tabs.get(0).tabList.items.first().selected = true;
        });

        $("#create-job-form").find('[name="jobName"]').on("input", function () {
            if ($("#create-job-form").find('[name="jobName"]').attr("aria-invalid") === "true") {
                $("#create-job-form").find('[name="jobName"]').parent().find(".coral-Form-fielderror").hide();
            }
        });

        $dialog.on("foundation-validation-invalid", function (event) {
            var target = $(event.target);
            target.parent().find(".coral-Form-fielderror").show();
        });
    }

})(jQuery(document), Granite.$);