;(function (document, $) {
    "use strict";

    function initializeValidators() {

        var ERROR_ICON_ID = 'smartling-validation-error-icon';
        var ERROR_TOOLTIP_ID = 'smartling-validation-error-tooltip';
        var SMARTLING_DIALOG_FOOTER_SELECTOR = '#smartling-job-dialog coral-dialog-footer'
        var SMARTLING_ERROR_ICON_CLASS = 'smartling-error-icon';

        function showError(message) {
            if (!message) {
                return;
            }

            if ($(SMARTLING_DIALOG_FOOTER_SELECTOR + ' #' + ERROR_ICON_ID).length === 0) {
                var icon = new Coral.Icon().set(
                    {
                        icon: 'alert',
                        size: 'L'
                    });
                icon = $(icon)
                    .attr('id', ERROR_ICON_ID)
                    .addClass(SMARTLING_ERROR_ICON_CLASS);
                var isStandaloneAemInstance = $(SMARTLING_DIALOG_FOOTER_SELECTOR).find('._coral-Button').length === 0;
                if (isStandaloneAemInstance) {
                    icon.css('margin-bottom', '1.5rem');
                }

                $(SMARTLING_DIALOG_FOOTER_SELECTOR).prepend(icon);
            }

            if ($(SMARTLING_DIALOG_FOOTER_SELECTOR + ' #' + ERROR_TOOLTIP_ID).length === 0) {
                var tooltip = new Coral.Tooltip().set({
                    content: {
                        innerHTML: message
                    },
                    variant: 'error',
                    placement: 'top',
                    target: "#" + ERROR_ICON_ID
                });
                tooltip = $(tooltip).attr('id', ERROR_TOOLTIP_ID);
                $(SMARTLING_DIALOG_FOOTER_SELECTOR).prepend(tooltip);
            }

            $(SMARTLING_DIALOG_FOOTER_SELECTOR + ' #' + ERROR_TOOLTIP_ID)[0].content.innerHTML = message;
            $(SMARTLING_DIALOG_FOOTER_SELECTOR + ' #' + ERROR_ICON_ID).show();
            $('#smartling-job-dialog').find('coral-tab[aria-selected="true"]').attr("invalid", true);
        }

        function hideErrorIcon() {
            $(SMARTLING_DIALOG_FOOTER_SELECTOR + ' #' + ERROR_ICON_ID).hide();
        }

        function processErrorResponse(response, reject) {
            var errorMessage = $.parseJSON(response.responseText).error.details[0];
            showError(errorMessage);

            window.smartling.touchui.jobDialog.processValidationError && window.smartling.touchui.jobDialog.processValidationError();

            reject(errorMessage);
        }

        var cloudConfigValidator = function() {
            new Promise(function(resolve, reject) {
                $.ajax({
                    url: '/services/smartling/cloud/config/validator',
                    data: {
                        payload: window.smartling.touchui.jobDialog.pagePath
                    },
                    success: resolve
                }).fail(function (response) {
                    processErrorResponse(response, reject);
                });
            });
        }

        var autoExecuteValidator = function () {
            return new Promise(function(resolve, reject) {
                $.ajax({
                    url: '/services/smartling/auto/execute/validator',
                    data: {
                        payload: window.smartling.touchui.jobDialog.pagePath
                    },
                    success: resolve
                }).fail(function (response) {
                    processErrorResponse(response, reject);
                });
            });
        }

        var targetLocalesValidator = function (form) {
            return new Promise(function(resolve, reject) {
                var targetLocalesElement;
                if (form.find('[name="jobUid"]').val()){
                    targetLocalesElement = $('#add-to-job-form [name="targetLocaleCodes"]');
                }else{
                    targetLocalesElement = $('#create-job-form [name="targetLocaleCodes"]');
                }
                $.ajax({
                    url: '/services/smartling/target/locales/validator',
                    data: {
                        payload: window.smartling.touchui.jobDialog.pagePath,
                        targetLocaleCodes: $.map(targetLocalesElement, function(element){return element.value})
                    },
                    success: resolve
                }).fail(function (response) {
                    processErrorResponse(response, reject);
                });
            });
        }

        var targetParentPathValidator = function (form) {
            return new Promise(function(resolve, reject) {
                var targetLocalesElement;
                if (form.find('[name="jobUid"]').val()){
                    targetLocalesElement = $('#add-to-job-form [name="targetLocaleCodes"]');
                }else{
                    targetLocalesElement = $('#create-job-form [name="targetLocaleCodes"]');
                }
                $.ajax({
                    url: '/services/smartling/page/parent/validator',
                    data: {
                        payload: window.smartling.touchui.jobDialog.pagePath,
                        targetLocaleCodes: $.map(targetLocalesElement, function(element){return element.value})
                    },
                    success: resolve
                }).fail(function (response) {
                    processErrorResponse(response, reject);
                });
            });
        }

        var cloudConfigChildpageValidator = function () {
            return new Promise(function(resolve, reject) {
                var isDeep = $(".coral-Panel.is-selected").find("coral-checkbox [name='isDeep']").is(':checked');

                $.ajax({
                    url: '/services/smartling/cloud/config/childpage/validator',
                    data: {
                        payload: window.smartling.touchui.jobDialog.pagePath,
                        isDeep: isDeep
                    },
                    success: resolve
                }).fail(function (response) {
                    processErrorResponse(response, reject);
                });
            });
        }

        var jobsNameValidator = function (form) {
            return new Promise(function (resolve, reject) {
                $.ajax({
                    url: '/services/smartling/jobs/name/validator',
                    data: {
                        payload: window.smartling.touchui.jobDialog.pagePath,
                        jobName: form.find('[name="jobName"]').val(),
                        jobUid: form.find('[name="jobUid"]').val()
                    },
                    success: resolve
                }).fail(function (response) {
                    var fieldWrapper = form.find('[name="jobName"]').parent();
                    var errorMessage = $.parseJSON(response.responseText).error.details[0];

                    fieldWrapper.find('#smartling-job-name-validator-icon').remove();
                    fieldWrapper.find('label[aria-describedby="smartling-job-name-validator-icon"]').remove();

                    var iconLabel = '<label class="coral-Form-errorlabel" aria-describedby="smartling-job-name-validator-icon" tabindex="0">' + errorMessage + '</label>';
                    var icon = new Coral.Icon().set({ icon: 'alert', size: 'S' });
                    icon = $(icon)
                        .addClass('coral-Form-fielderror')
                        .attr('id','smartling-job-name-validator-icon')
                        .addClass(SMARTLING_ERROR_ICON_CLASS);

                    fieldWrapper.append(icon);
                    fieldWrapper.append(iconLabel);

                    $('#create-job-form [name="jobName"]').attr('invalid',true);
                    $('#smartling-job-dialog').find('coral-tab[aria-selected="true"]').attr("invalid", true);

                    reject('Job name is not unique');
                });
            });
        }

        var cloudConfigMatchValidator = function (form) {
            return new Promise(function(resolve, reject) {
                var targetLocalesElement;
                if (form.find('[name="jobUid"]').val()){
                    targetLocalesElement = $('#add-to-job-form [name="targetLocaleCodes"]');
                }else{
                    targetLocalesElement = $('#create-job-form [name="targetLocaleCodes"]');
                }
                var payloadPath = window.smartling.touchui.jobDialog.pagePath;
                payloadPath = payloadPath.indexOf(';') > 0
                    ? payloadPath.substr(0, payloadPath.indexOf(';'))
                    : payloadPath;
                $.ajax({
                    url: '/services/smartling/config/match/validator',
                    data: {
                        payload: payloadPath,
                        targetLocaleCodes: $.map(targetLocalesElement, function(element){return element.value})
                    },
                    success: resolve
                }).fail(function (response) {
                    processErrorResponse(response, reject);
                });
            });
        }

        $('#create-job-form [name="jobName"]').on('change', function(){
            $('#create-job-form label[aria-describedby="smartling-job-name-validator-icon"]').remove();
        });

        $("#create-job-form").submit(function (e) {
            hideErrorIcon();

            var form = $("#create-job-form");
            if( form.data("valid") === true) {
                return;
            }

            e.preventDefault();

            Promise.all([cloudConfigValidator(form),
                autoExecuteValidator(form),
                targetLocalesValidator(form),
                cloudConfigMatchValidator(form),
                cloudConfigChildpageValidator(form),
                jobsNameValidator(form),
                targetParentPathValidator(form)]).then(function(values) {
                form.data("valid", true);
                form.submit();
                form.data("valid", false);
            });

            return false;
        });

        $("#add-to-job-form").submit(function (e) {
            hideErrorIcon();

            var form = $("#add-to-job-form");
            if( form.data("valid") === true) {
                return;
            }

            e.preventDefault();

            Promise.all([cloudConfigValidator(form),
                autoExecuteValidator(form),
                targetLocalesValidator(form),
                cloudConfigMatchValidator(form),
                cloudConfigChildpageValidator(form),
                targetParentPathValidator(form)]).then(function(values) {
                form.data("valid", true);
                form.submit();
                form.data("valid", false);
            });

            return false;
        });
    }

    $(document).ready(initializeValidators);

})(jQuery(document), Granite.$);